#include <inarm.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"



CSP_AMC_T * const AMC = (CSP_AMC_T *)AMC_BASE_ADDRESS;

#define TEST_NUMB   10
#define TEST_ERR    1
#define TEST_OK     0

const char NewLine[]  = "\n";
const char StrPass[]  = "Pass";
const char StrFault[] = "Fault !!!!!!!!!!!!!!!!!!!!!!!";

volatile unsigned long   * pLong;
volatile unsigned short  * pShort;
volatile unsigned char   * pByte;

unsigned char Status;
unsigned long i;
unsigned long j;
unsigned long line;
unsigned long mask;

const unsigned long RamTest [TEST_NUMB] = {
/*
  0xFFFFFFFF,
  0xFFFFFFFE,
  0xFFFFFFFC,
  0xFFFFFFF8,
  0xFFFFFFF0,

  0xFFFFFFEF,
  0xFFFFFFCF,
  0xFFFFFF8F,
  0xFFFFFF0F,

  0xFFFFFEFF,
  0xFFFFFCFF,
  0xFFFFF8FF,
  0xFFFFF0FF,

  0xFFFFEFFF,
  0xFFFFCFFF,
  0xFFFF8FFF,
  0xFFFF0FFF,

  0xFFFEFFFF,
  0xFFFCFFFF,
  0xFFF8FFFF,
  0xFFF0FFFF,

  0xFFEFFFFF,
  0xFFCFFFFF,
  0xFF8FFFFF,
  0xFF0FFFFF,

  0xFEFFFFFF,
  0xFCFFFFFF,
  0xF8FFFFFF,
  0xF0FFFFFF,

  0xEFFFFFFF,
  0xCFFFFFFF,
  0x8FFFFFFF,
  0x0FFFFFFF,

  ~0xFFFFFFFF,
  ~0xFFFFFFFE,
  ~0xFFFFFFFC,
  ~0xFFFFFFF8,
  ~0xFFFFFFF0,

  ~0xFFFFFFEF,
  ~0xFFFFFFCF,
  ~0xFFFFFF8F,
  ~0xFFFFFF0F,

  ~0xFFFFFEFF,
  ~0xFFFFFCFF,
  ~0xFFFFF8FF,
  ~0xFFFFF0FF,

  ~0xFFFFEFFF,
  ~0xFFFFCFFF,
  ~0xFFFF8FFF,
  ~0xFFFF0FFF,

  ~0xFFFEFFFF,
  ~0xFFFCFFFF,
  ~0xFFF8FFFF,
  ~0xFFF0FFFF,

  ~0xFFEFFFFF,
  ~0xFFCFFFFF,
  ~0xFF8FFFFF,
  ~0xFF0FFFFF,

  ~0xFEFFFFFF,
  ~0xFCFFFFFF,
  ~0xF8FFFFFF,
  ~0xF0FFFFFF,

  ~0xEFFFFFFF,
  ~0xCFFFFFFF,
  ~0x8FFFFFFF,
  ~0x0FFFFFFF,
*/
  0xFFFFFF55,
  0xFFFF55FF,
  0xFF55FFFF,
  0x55FFFFFF,

  ~0xFFFFFF55,
  ~0xFFFF55FF,
  ~0xFF55FFFF,
  ~0x55FFFFFF,

  0xAAAAAAAA,
  0x55555555,
};

void TestFault(void) {

  //printf(StrFault);
  //printf(NewLine);

  UARTString((unsigned char*)StrFault);
  UARTString((unsigned char*)NewLine);
}

void TestPass(void) {

  //printf(StrPass);
  //printf(NewLine);

  UARTString((unsigned char*)StrPass);
  UARTString((unsigned char*)NewLine);
}


void InitAMC(void) {

  // Init AMC
  AMC->MCR = ALE_2;

  AMC->CSR[0] = (BA & 0x40000000) | // offset 2MB
                CSEN_ENABLE       | // CS Enable
                BAT_SELECT        | // Byte access enable
                TDF_7             | // Data Float Output Time
                PAGES_4           | // Page size 4MB
                WSE_ENABLE        | // Wait statet enable
                NWS_8             | // 8 wait states
                DBW_16;             // Data Bus Width 16 bits

  AMC->CSR[1] = (BA & 0x40400000) | // offset 2MB
                CSEN_ENABLE       | // CS Enable
                BAT_SELECT        | // Byte access enable
                TDF_0             | // Data Float Output Time
                PAGES_4           | // Page size 4MB
                WSE_ENABLE        | // Wait statet enable
                NWS_3             | // 8 wait states
                DBW_16;             // Data Bus Width 16 bits

  AMC->CSR[2] = (BA & 0x40800000) | // offset 2MB
                CSEN_ENABLE       | // CS Enable
                BAT_SELECT        | // Byte access enable
                TDF_0             | // Data Float Output Time
                PAGES_4           | // Page size 4MB
                WSE_ENABLE        | // Wait statet enable
                NWS_3             | // 8 wait states
                DBW_16;             // Data Bus Width 16 bits

}


char TestRam(void) {


  UARTInitialize(9600);

  // printf("\n************ RAM Test begin ************\n");
  UARTString("\n\r\n\r************ RAM Test begin ************");

  Status = 0;

  // Data accses test
  // printf("1. Data access test - ");
  UARTString("\n\r1. Data access test - ");

  pLong  = (unsigned long  *)0x40600000;
  pShort = (unsigned short *)0x40600000;
  pByte  = (unsigned char  *)0x40600000;

  *pLong = 0x12345678;

  if(*pShort == 0x5678 && *++pShort == 0x1234) {

    *pShort   = 0x5678;
    *--pShort = 0x1234;

    if(*pLong == 0x56781234) {

      if(*pByte == 0x34 && *++pByte == 0x12) {
        TestPass();
      }
      else {
        TestFault();
        return TEST_ERR;
      }
    }
    else {
      TestFault();
      return TEST_ERR;
    }
  }
  else {
    TestFault();
    return TEST_ERR;
  }


  // Address bus test - low 2 MB
  // printf("2. Address bus test - ");
  UARTString("\n\r2. Address bus test - ");
  pLong  = (unsigned long  *)0x40600000;
  for(i = 0; i < 0x00400000; i+= 4, ++pLong)
  {
    *pLong = i;
  }

  pLong  = (unsigned long  *)0x40600000;
  for(i = 0; i < 0x00400000; i+= 4)
  {
    if(*pLong++ != i)
    {
      Status = 1;
      //printf(NewLine);
      //printf("Address bus error at address 0x%X",i);
      UARTString("\n\r Address bus error at address: (dont work at the moment)");
    }
  }

  if(!Status)
  {
    TestPass();
  }
  else
  {
    // printf(NewLine);
    UARTString((unsigned char*)NewLine);
    TestFault();
    return TEST_ERR;
  }


  // Data Bus test
  // printf("3. Data bus test - ");
  UARTString("\n\r3. Data bus test - ");
  pLong  = (unsigned long  *)0x40600000;
  for(i = 0; i < TEST_NUMB; ++i)
  {
    *pLong = RamTest[i];
    if(*pLong != RamTest[i])
    {
      Status = 1;
      // printf(NewLine);
      // printf("Data bus error at pattern 0x%X",RamTest[i]);

      UARTString("\n\rData bus error at pattern: (dont work at the moment)");

    }
  }
  if(!Status)
  {
    TestPass();
  }
  else
  {
    // printf(NewLine);
    UARTString((unsigned char*)NewLine);
    TestFault();
    return TEST_ERR;
  }




  // Memory test
  // printf("4. Memory device test");
  // printf(NewLine);
  // printf("  ");
  UARTString("\n\r4. Memory device test");
  UARTString("  ");

  unsigned long Tmp;
  for(j = 0,line = 0; j < TEST_NUMB; ++j)
  {
    pLong  = (unsigned long  *)0x40600000;
    mask = RamTest[j];
    for(i = 0; i < 0x00400000; i+= 4,++pLong)
    {
      Tmp = i & 0xFF;
      *pLong = mask ^ (Tmp | ((Tmp<<8)&0xFF00) | ((Tmp<<16)&0xFF0000) | ((Tmp<<24)&0xFF000000));
    }
    pLong  = (unsigned long  *)0x40600000;

    // printf("*");
    UARTString("*");

    if(++line > 40)
    {
      line = 0;
      //printf(NewLine);
      //printf("  ");

      UARTString("  ");
    }
    for(i = 0; i < 0x00400000; i+= 4,++pLong)
    {
      Tmp = i & 0xFF;
      Tmp = (Tmp | ((Tmp<<8)&0xFF00) | ((Tmp<<16)&0xFF0000) | ((Tmp<<24)&0xFF000000));
      if(*pLong != (mask ^ Tmp))
      {
        Status = 1;
        // printf(NewLine);
        // printf("Memory error at address 0x%X with pattern 0x%X",i,mask ^ i);

        UARTString("\n\rMemory error at address XX with pattern YY (dont work at the moment)");
      }
    }
  }
  if(!Status)
  {
    // printf(NewLine);
    TestPass();
  }
  else
  {
    // printf(NewLine);
    TestFault();

    return TEST_ERR;
  }

  return TEST_OK;
}

