#include <inarm.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"

void Delay(unsigned long d) {
  volatile unsigned long dly = d;
  while(dly--);
}

// ADC
CSP_ADC8C_T * const ADC8C_0     = ((CSP_ADC8C_T *)   ADC0_BASE_ADDRESS);
CSP_ADC8C_T * const ADC8C_1     = ((CSP_ADC8C_T *)   ADC1_BASE_ADDRESS);

unsigned int adc_data = 0;
char state_adc_err = 0;
unsigned char c = 0;

// UPIO
CSP_PIO_T * const UPIO            = ((CSP_PIO_T *)   UPIO_BASE_ADDRESS);

char state_upio_vcc = 0;
char state_upio_gnd = 0;
char state_upio_pin = 100;

char state_ext_vcc  = 0;
char state_ext_gnd  = 0;
char state_ext_pin  = 100;


const unsigned int mask_upio    = 0xBFFFFFF3&(~BIT31);   // pullup UPIO31

unsigned int l=0;

// EXT
extern CSP_USART_T * const USART0;
extern CSP_USART_T * const USART1;


CSP_GPT_T * const GPT0CH0           = ((CSP_GPT_T *)   GPT0CH0_BASE_ADDRESS);
CSP_GPT_T * const GPT0CH1           = ((CSP_GPT_T *)   GPT0CH1_BASE_ADDRESS);
CSP_GPT_T * const GPT0CH2           = ((CSP_GPT_T *)   GPT0CH2_BASE_ADDRESS);
CSP_GPT_T * const GPT1CH0           = ((CSP_GPT_T *)   GPT1CH0_BASE_ADDRESS);

unsigned int capt0_val=0;
unsigned int capt1_val=0;
char pwm_cap_err=0;

// UEXT
unsigned char state_uext_vcc = 0;
unsigned char state_uext_gnd = 0;
unsigned char state_uext_pin = 100;

extern CSP_SPI_T * const SPI0;

void InitAdc0(void) {

  //// ADC0 ////
  // Reset ADC
  ADC8C_0->CR = SWRST;

  // Enable clock for ADC
  ADC8C_0->ECR = ADC;

  // CoreClk=30MHz, ADC~=700KHz => PREVAL = 4*12, one conversion
  ADC8C_0->MR = STARTUPTIME(120) | PRLVAL(12);

  // Enable ADC
  ADC8C_0->CR |= ADCEN;

}


unsigned int GetAdc0Value(unsigned char chanel) {

  // Set desired chanel
  ADC8C_0->CMR = chanel;

  // wait until ready
  while(ADC8C_0->CSR&READY);

  // Start conversion
  ADC8C_0->CR |= START;

  // wait until conversion finish
  while(ADC8C_0->CSR&EOC);

  Delay(1000);

  // return converted value
  return ADC8C_0->DR;
}


void InitAdc1(void) {

  //// ADC0 ////
  // Reset ADC
  ADC8C_1->CR = SWRST;

  // Enable clock for ADC
  ADC8C_1->ECR = ADC;

  // CoreClk=30MHz, ADC~=700KHz => PREVAL = 4*12, one conversion
  ADC8C_1->MR = STARTUPTIME(120) | PRLVAL(12);

  // Enable ADC
  ADC8C_1->CR |= ADCEN;

}


unsigned int GetAdc1Value(unsigned char chanel) {

  // Set desired chanel
  ADC8C_1->CMR = chanel;

  // wait until ready
  while(ADC8C_1->CSR&READY);

  // Start conversion
  ADC8C_1->CR |= START;

  // wait until conversion finish
  while(ADC8C_1->CSR&EOC);

  Delay(1000);

  // return converted value
  return ADC8C_1->DR;
}


void Stop(void) {

  UARTInitialize(9600);
  UARTString("\n\r\n\r Error!!!!");

  if(state_adc_err)  UARTString("\n\r ADC");

  if(state_upio_vcc)      UARTString("\n\r Upio VCC");
  if(state_upio_gnd)      UARTString("\n\r Upio GND");
  if(state_upio_pin!=100) {
      UARTString("\n\r Upio PIN");
      UARTWriteChar(state_upio_pin/10+48);
      UARTWriteChar(state_upio_pin%10+48);
  }


  if(state_ext_vcc)       UARTString("\n\r Ext VCC");
  if(state_ext_gnd)       UARTString("\n\r Ext GND");
  if(state_ext_pin!=100) {
      UARTString("\n\r Ext PIN");
      UARTWriteChar(state_ext_pin/10+48);
      UARTWriteChar(state_ext_pin%10+48);
  }

  if(pwm_cap_err)         UARTString("\n\r Pwm Cap Error");

  if(state_uext_vcc)      UARTString("\n\r Uext VCC");
  if(state_uext_gnd)      UARTString("\n\r Uext GND");
  if(state_uext_pin!=100) {
      UARTString("\n\r Uext PIN");
      UARTWriteChar(state_uext_pin/10+48);
      UARTWriteChar(state_uext_pin%10+48);
  }

  while(1);

}

unsigned char TestBoard(void) {

  // Test ADC //----------------------------------------------------------------
  adc_data = 0;

  // Init Adc0 and Adc1
  InitAdc0();
  adc_data = GetAdc0Value(0);       if((adc_data>20))                   { state_adc_err = 1; Stop(); }
  adc_data = GetAdc0Value(1);       if((adc_data<100)||(adc_data>150))  { state_adc_err = 1; Stop(); }
  adc_data = GetAdc0Value(2);       if((adc_data<230)||(adc_data>280))  { state_adc_err = 1; Stop(); }
  adc_data = GetAdc0Value(3);       if((adc_data<350)||(adc_data>410))  { state_adc_err = 1; Stop(); }
  adc_data = GetAdc0Value(4);       if((adc_data<480)||(adc_data>540))  { state_adc_err = 1; Stop(); }
  adc_data = GetAdc0Value(5);       if((adc_data<610)||(adc_data>670))  { state_adc_err = 1; Stop(); }
  adc_data = GetAdc0Value(6);       if((adc_data<740)||(adc_data>800))  { state_adc_err = 1; Stop(); }
  adc_data = GetAdc0Value(7);       if((adc_data<860)||(adc_data>930))  { state_adc_err = 1; Stop(); }

  InitAdc1();
  adc_data = GetAdc1Value(0);       if((adc_data>20))                   { state_adc_err = 1; Stop(); }
  adc_data = GetAdc1Value(1);       if((adc_data<100)||(adc_data>150))  { state_adc_err = 1; Stop(); }
  adc_data = GetAdc1Value(2);       if((adc_data<230)||(adc_data>280))  { state_adc_err = 1; Stop(); }
  adc_data = GetAdc1Value(3);       if((adc_data<350)||(adc_data>410))  { state_adc_err = 1; Stop(); }
  adc_data = GetAdc1Value(4);       if((adc_data<480)||(adc_data>540))  { state_adc_err = 1; Stop(); }
  adc_data = GetAdc1Value(5);       if((adc_data<610)||(adc_data>670))  { state_adc_err = 1; Stop(); }
  adc_data = GetAdc1Value(6);       if((adc_data<740)||(adc_data>800))  { state_adc_err = 1; Stop(); }
  adc_data = GetAdc1Value(7);       if((adc_data<860)||(adc_data>930))  { state_adc_err = 1; Stop(); }

  // End Test ADC //------------------------------------------------------------

  // Pretest setting

  // EXT /////////////////////
  // Enable clock for PIO
  UPIO->ECR = 0x1;

  // all as input
  UPIO->ODR    = 0xffffffff;   // All as input

  // Enable clock
  GPT0CH0->ECR = 0x1;
  GPT0CH1->ECR = 0x1;
  GPT0CH2->ECR = 0x1;
  GPT1CH0->ECR = 0x1;
  USART0->ECR = 0x1;

  // GPIO
  // USART0 and USART1 (SCK as GPIO)
  USART0->PER = SCK;
  USART1->PER = SCK;
  // GPT0CH0 and GPT0CH0 (A0, A1, CLK as GPIO)
  GPT0CH0->PER = TIOB + TIOA + TCLK;
  GPT0CH1->PER = TIOB + TIOA + TCLK;
  GPT0CH2->PER = TIOB + TIOA + TCLK;
  GPT1CH0->PER = TIOB + TIOA + TCLK;

  // Input
  USART0->ODR = SCK;
  USART1->ODR = SCK;
  GPT0CH0->ODR = TIOB + TIOA + TCLK;
  GPT0CH1->ODR = TIOB + TIOA + TCLK;
  GPT0CH2->ODR = TIOB + TIOA + TCLK;
  GPT1CH0->ODR = TIOB + TIOA + TCLK;

  // Pull up high
  USART0->SODR  = SCK;    // Set
  USART0->OER   = SCK;    // Output

  // UEXT //////////
  // Enable clock
  SPI0->ECR = 0x1;
  // GPIO
  USART1->PER = TXD + RXD;
  SPI0->PER = SPCK + MISO + MOSI + NPCS0 + NPCS2 + NPCS3;

  // Input
  USART1->ODR = TXD + RXD;
  SPI0->ODR = SPCK + MISO + MOSI + NPCS0 + NPCS2 + NPCS3;

  // Pull up - high
  SPI0->SODR = MISO;
  SPI0->OER = MISO;

  // Test UPIO //---------------------------------------------------------------


  //// Test for VCC /////---------------------------------------
  // Enable clock for PIO
  UPIO->ECR = 0x1;

  // all as input
  UPIO->ODR    = 0xffffffff;   // All as input

  // just simple delay
  Delay(1000);

  // pull up
  UPIO->CODR   = BIT31; // low
  UPIO->OER    = BIT31; // output

  // just simple delay
  Delay(1000);

  // check for high
  if((UPIO->PDSR&mask_upio) != 0x0) {
    state_upio_vcc = 1;
    Stop();
  }


  //// End test for VCC /////---------------------------------------

  //// Test for GND /////-----------------------------------

  // all as input
  UPIO->ODR    = 0xffffffff;   // All as input

  // just simple delay
  Delay(1000);

  // pull up
  UPIO->SODR   = BIT31; // high
  UPIO->OER    = BIT31; // output

  // just simple delay
  Delay(1000);

  // check for low
  if((UPIO->PDSR|(~mask_upio)) != 0xFFFFFFFF) {
    state_upio_gnd = 1;
    Stop();
  }

  //// End test for GND /////-----------------------------------

  ///// Running zero ///////-----------------------------------

  ///// for Port0

  // all as input
  UPIO->ODR    = 0xffffffff;   // All as input

  // just simple delay
  Delay(1000);

  // pull up
  UPIO->SODR   = BIT31; // high
  UPIO->OER    = BIT31; // output

  // just simple delay
  Delay(1000);

  // loop for port a
  for(l=0; l<32; l++) {

    // this port is not tested
    if(!((mask_upio)&(1<<l))) continue;

    UPIO->SODR   = 0xffffffff;
    UPIO->CODR   = (1<<l);

    UPIO->ODR    = 0xffffffff;
    UPIO->OER    = (1<<l) | BIT31;

    // just simple delay
    Delay(1000);

    // check for other zero at port A
    if((UPIO->PDSR|(~mask_upio)) != ((0xFFFFFFFF)&(~(1<<l)))) {
      state_upio_pin = l+20;
      Stop();
    }


    // Check for low at EXT
    if( (USART1->PDSR&SCK!=SCK) || ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
        ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
        ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
        ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

        state_upio_pin = l;
        Stop();
    }

    // check for low at UEXT
    if( (USART1->PDSR&(TXD|RXD)!=(TXD|RXD)) ||
        ((SPI0->PDSR&(SPCK|MOSI|NPCS0|NPCS2|NPCS3))!=(SPCK|MOSI|NPCS0|NPCS2|NPCS3)) ) {

        state_upio_pin = l;
        Stop();
    }

  }
  // End Test UPIO //-----------------------------------------------------------

  // Test UEXT //---------------------------------------------------------------
  // GPIO
  USART1->PER = TXD + RXD;
  SPI0->PER = SPCK + MISO + MOSI + NPCS0 + NPCS2 + NPCS3;

  // Input
  USART1->ODR = TXD + RXD;
  SPI0->ODR = SPCK + MISO + MOSI + NPCS0 + NPCS2 + NPCS3;

  // Pull up - high
  SPI0->SODR = MISO;
  SPI0->OER = MISO;

  // check for low
  if( (USART1->PDSR&(TXD|RXD)!=(TXD|RXD)) ||
      ((SPI0->PDSR&(SPCK|MOSI|NPCS0|NPCS2|NPCS3))!=(SPCK|MOSI|NPCS0|NPCS2|NPCS3)) ) {

      state_uext_gnd = 1;
      Stop();
  }

  // Pull up - low
  SPI0->CODR = MISO;
  SPI0->OER = MISO;

  // check for high
  if( (USART1->PDSR&(TXD|RXD)!=(0)) ||
      ((SPI0->PDSR&(SPCK|MOSI|NPCS0|NPCS2|NPCS3))!=(0)) ) {

      state_uext_vcc = 1;
      Stop();
  }

  // Runnign zero
  // Pull up - high
  SPI0->SODR = MISO;
  SPI0->OER = MISO;

  // TXD -> low
  USART1->CODR = TXD;
  USART1->OER = TXD;
  // check for low
  if( (USART1->PDSR&(RXD)!=(RXD)) ||
      ((SPI0->PDSR&(SPCK|MOSI|NPCS0|NPCS2|NPCS3))!=(SPCK|MOSI|NPCS0|NPCS2|NPCS3)) ) {

      state_uext_pin = 1;
      Stop();
  }
  USART1->ODR = TXD;

  // RXD -> low
  USART1->CODR = RXD;
  USART1->OER = RXD;
  // check for low
  if( (USART1->PDSR&(TXD)!=(TXD)) ||
      ((SPI0->PDSR&(SPCK|MOSI|NPCS0|NPCS2|NPCS3))!=(SPCK|MOSI|NPCS0|NPCS2|NPCS3)) ) {

      state_uext_pin = 2;
      Stop();
  }
  USART1->ODR = RXD;

  //SPCK -> low
  SPI0->CODR = SPCK;
  SPI0->OER = SPCK;
  // check for low
  if( (USART1->PDSR&(TXD|RXD)!=(TXD|RXD)) ||
      ((SPI0->PDSR&(MOSI|NPCS0|NPCS2|NPCS3))!=(MOSI|NPCS0|NPCS2|NPCS3)) ) {

      state_uext_pin = 3;
      Stop();
  }

  SPI0->ODR = SPCK;

  //MOSI -> low
  SPI0->CODR = MOSI;
  SPI0->OER = MOSI;
  // check for low
  if( (USART1->PDSR&(TXD|RXD)!=(TXD|RXD)) ||
      ((SPI0->PDSR&(SPCK|NPCS0|NPCS2|NPCS3))!=(SPCK|NPCS0|NPCS2|NPCS3)) ) {

      state_uext_pin = 4;
      Stop();
  }

  SPI0->ODR = MOSI;

  //NPCS0 -> low
  SPI0->CODR = NPCS0;
  SPI0->OER = NPCS0;
  // check for low
  if( (USART1->PDSR&(TXD|RXD)!=(TXD|RXD)) ||
      ((SPI0->PDSR&(SPCK|MOSI|NPCS2|NPCS3))!=(SPCK|MOSI|NPCS2|NPCS3)) ) {

      state_uext_pin = 5;
      Stop();
  }

  SPI0->ODR = NPCS0;


  //NPCS2 -> low
  SPI0->CODR = NPCS2;
  SPI0->OER = NPCS2;
  // check for low
  if( (USART1->PDSR&(TXD|RXD)!=(TXD|RXD)) ||
      ((SPI0->PDSR&(SPCK|MOSI|NPCS0|NPCS3))!=(SPCK|MOSI|NPCS0|NPCS3)) ) {

      state_uext_pin = 6;
      Stop();
  }

  SPI0->ODR = NPCS2;

  //NPCS2 -> low
  SPI0->CODR = NPCS3;
  SPI0->OER = NPCS3;
  // check for low
  if( (USART1->PDSR&(TXD|RXD)!=(TXD|RXD)) ||
      ((SPI0->PDSR&(SPCK|MOSI|NPCS0|NPCS2))!=(SPCK|MOSI|NPCS0|NPCS2)) ) {

      state_uext_pin = 7;
      Stop();
  }

  SPI0->ODR = NPCS3;

  // Test UEXT //---------------------------------------------------------------


  // Test EXT //----------------------------------------------------------------
  // Enable clock
  GPT0CH0->ECR = 0x1;
  GPT0CH1->ECR = 0x1;
  GPT0CH2->ECR = 0x1;
  GPT1CH0->ECR = 0x1;
  USART0->ECR = 0x1;

  // GPIO
  // USART0 and USART1 (SCK as GPIO)
  USART0->PER = SCK;
  USART1->PER = SCK;
  // GPT0CH0 and GPT0CH0 (A0, A1, CLK as GPIO)
  GPT0CH0->PER = TIOB + TIOA + TCLK;
  GPT0CH1->PER = TIOB + TIOA + TCLK;
  GPT0CH2->PER = TIOB + TIOA + TCLK;
  GPT1CH0->PER = TIOB + TIOA + TCLK;

  // Input
  USART0->ODR = SCK;
  USART1->ODR = SCK;
  GPT0CH0->ODR = TIOB + TIOA + TCLK;
  GPT0CH1->ODR = TIOB + TIOA + TCLK;
  GPT0CH2->ODR = TIOB + TIOA + TCLK;
  GPT1CH0->ODR = TIOB + TIOA + TCLK;

  // Pull up high
  USART0->SODR  = SCK;    // Set
  USART0->OER   = SCK;    // Output

  // just simple delay
  Delay(1000);

  // Check for low
  if( (USART1->PDSR&SCK!=SCK) || ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_vcc = 0;
  }

  USART0->CODR  = SCK;    // Clear
  USART0->OER   = SCK;    // Output

  // just simple delay
  Delay(1000);

  // Check for high
  if( (USART1->PDSR&SCK!=(0)) || ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(0)) ||
      ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(0)) ||
      ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(0)) ||
      ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(0)) ) {

       state_ext_gnd = 0;
  }

  // Running Zero
  //
  // Pull up high
  USART0->SODR  = SCK;    // Set
  USART0->OER   = SCK;    // Output
  //
  USART1->CODR  = SCK;    // Clear
  USART1->OER   = SCK;    // Output

  // just simple delay
  Delay(1000);

  // Check for low
  if( ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_pin = 0;
      Stop();
   }


  USART1->ODR   = SCK;    // Input


  // GPT0CH0
  GPT0CH0->CODR = TIOA;   // Clear
  GPT0CH0->OER  = TIOA;   // Output

  // just simple delay
  Delay(1000);


  // Check for low
  if( (USART1->PDSR&SCK!=SCK) || ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TCLK)) ||
      ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_pin = 1;
      Stop();
  }

  GPT0CH0->ODR  = TIOA;   // Input


  GPT0CH0->CODR = TIOB;   // Clear
  GPT0CH0->OER  = TIOB;   // Output

  // just simple delay
  Delay(1000);


  // Check for low
  if( (USART1->PDSR&SCK!=SCK) || ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOA|TCLK)) ||
      ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_pin = 2;
      Stop();
  }

  GPT0CH0->ODR  = TIOB;   // Input


  GPT0CH0->CODR = TCLK;   // Clear
  GPT0CH0->OER  = TCLK;   // Output

  // just simple delay
  Delay(1000);


  // Check for low
  if( (USART1->PDSR&SCK!=SCK) || ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA)) ||
      ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_pin = 3;
      Stop();
  }

  GPT0CH0->ODR  = TCLK;   // Input

  // GPT0CH1
  GPT0CH1->CODR = TIOA;   // Clear
  GPT0CH1->OER  = TIOA;   // Output

  // just simple delay
  Delay(1000);


  // Check for low
  if( (USART1->PDSR&SCK!=SCK) || ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TCLK)) ||
      ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_pin = 4;
      Stop();
  }

  GPT0CH1->ODR  = TIOA;   // Input


  GPT0CH1->CODR = TIOB;   // Clear
  GPT0CH1->OER  = TIOB;   // Output

  // just simple delay
  Delay(1000);


  // Check for low
  if( (USART1->PDSR&SCK!=SCK) || ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOA|TCLK)) ||
      ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_pin = 5;
      Stop();
  }

  GPT0CH1->ODR  = TIOB;   // Input


  GPT0CH1->CODR = TCLK;   // Clear
  GPT0CH1->OER  = TCLK;   // Output

  // just simple delay
  Delay(1000);


  // Check for low
  if( (USART1->PDSR&SCK!=SCK) || ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA)) ||
      ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_pin = 6;
      Stop();
  }

  GPT0CH1->ODR  = TCLK;   // Input


  // GPT0CH2
  GPT0CH2->CODR = TIOA;   // Clear
  GPT0CH2->OER  = TIOA;   // Output

  // just simple delay
  Delay(1000);


  // Check for low
  if( (USART1->PDSR&SCK!=SCK) || ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TCLK)) ||
      ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_pin = 7;
      Stop();
  }

  GPT0CH2->ODR  = TIOA;   // Input


  GPT0CH2->CODR = TIOB;   // Clear
  GPT0CH2->OER  = TIOB;   // Output

  // just simple delay
  Delay(1000);


  // Check for low
  if( (USART1->PDSR&SCK!=SCK) || ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOA|TCLK)) ||
      ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_pin = 8;
      Stop();
  }

  GPT0CH2->ODR  = TIOB;   // Input


  GPT0CH2->CODR = TCLK;   // Clear
  GPT0CH2->OER  = TCLK;   // Output

  // just simple delay
  Delay(1000);


  // Check for low
  if( (USART1->PDSR&SCK!=SCK) || ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA)) ||
      ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_pin = 9;
      Stop();
  }

  GPT0CH2->ODR  = TCLK;   // Input

  // GPT1CH0
  GPT1CH0->CODR = TIOA;   // Clear
  GPT1CH0->OER  = TIOA;   // Output

  // just simple delay
  Delay(1000);


  // Check for low
  if( (USART1->PDSR&SCK!=SCK) || ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TCLK)) ||
      ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_pin = 10;
      Stop();
  }

  GPT1CH0->ODR  = TIOA;   // Input


  GPT1CH0->CODR = TIOB;   // Clear
  GPT1CH0->OER  = TIOB;   // Output

  // just simple delay
  Delay(1000);


  // Check for low
  if( (USART1->PDSR&SCK!=SCK) || ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOA|TCLK)) ||
      ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_pin = 11;
      Stop();
  }

  GPT1CH0->ODR  = TIOB;   // Input


  GPT1CH0->CODR = TCLK;   // Clear
  GPT1CH0->OER  = TCLK;   // Output

  // just simple delay
  Delay(1000);


  // Check for low
  if( (USART1->PDSR&SCK!=SCK) || ((GPT1CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA)) ||
      ((GPT0CH1->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH2->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ||
      ((GPT0CH0->PDSR&(TIOB|TIOA|TCLK))!=(TIOB|TIOA|TCLK)) ) {

      state_ext_pin = 12;
      Stop();
  }

  GPT1CH0->ODR  = TCLK;   // Input


  // Test PWM and CAPT---------------------------------------------------------
  capt0_val = 0;
  capt1_val = 0;

  // Init Capt
  InitCapt0();
  InitCapt1();

  // Init PWM
  InitPwm0();
  InitPwm1();
  InitPwm2();

  // Pwm0 -> LED must blink
  Pwm0Enable();
  Pwm1Disable();
  Pwm2Disable();
  Delay(1000);

  // Test for short with CAPT0 and CAPT1
  // StarMeasureCapt0();
  // StarMeasureCapt1();
  // Delay(1000);

  capt0_val=GetCapt0Value();
  capt1_val=GetCapt1Value();

  if((capt0_val!=0)||(capt0_val!=0)) {
    pwm_cap_err=1;
    Stop();
  }

  Pwm1Enable();
  Pwm2Disable();
  Delay(1000);

  //StarMeasureCapt0();
  //StarMeasureCapt1();
  //Delay(1000);

  capt0_val=GetCapt0Value();
  capt1_val=GetCapt1Value();

  if((capt0_val<1000)||(capt1_val!=0)) {
    pwm_cap_err=1;
    Stop();
  }

  Pwm1Disable();
  Pwm2Enable();

  Delay(1000);

  //StarMeasureCapt0();
  //StarMeasureCapt1();
  //Delay(1000);

  capt0_val=GetCapt0Value();
  capt1_val=GetCapt1Value();

  if((capt0_val!=0)||(capt1_val<1000)) {
     pwm_cap_err=1;
     Stop();
  }

//  // djust for test
//  UART1Initialize(9600);
//  UART1WriteChar(0x55);
//
//  while(c!=0x55) {
//    UART1WriteChar(0x55);
//    c = UART1ReadChar_nonstop();
//  }



  // End Test EXT //------------------------------------------------------------

  return 0;
}



